from django.contrib.auth import get_user_model
from django.db import models

from tickets.models import Ticket


class Company(models.Model):
    """
    Company - это таблица с партнерами сервисов. Партнер сервиса -
    юридическое лицо или ИП, которое непосредственно организует работу с
    исполнителями и отвечает перед клиентами и перед сервисом за качество
    оказанных услуг. Юридически это лица, фактически оказывающие услуги
    по договору.
    """

    id = models.IntegerField(
        primary_key=True
    )

    # ID METASERVICE
    # FIXME: id_metaservice должно принимать текущий идентификатор метасервиса, на котором происходит создание записи
    id_metaservice = models.PositiveIntegerField(
        default=1,
        help_text="Уникальный идентификатор метасервиса, необходимый для синхронизации данных. Один и тот же провайдер может быть для нескольких метасервисов, соответственно если происходят изменения в одном, то либо форсируется изменение во всех (если возможно), либо снимается is_global. Соответственно при изменении is_global в true должно происходить согласование с остальными копиями в других сервисах. Нужен в том числе для того, чтобы выяснять, в каких еще сервисах есть этот провайдер."
    )

    # DATA
    requirements = models.CharField(
        "Ограничения",
        max_length=150,
        default="DS01y04y05y06y07yMPUCS13MPUB",
        help_text="Код необходимый для того, чтобы ресурс мог стать активным"
    )
    status = models.CharField(
        "Статус",
        max_length=150,
        default="deactivated",
        help_text="Cтатус обработки заявки"
    )
    legal_name = models.CharField(
        "Название организации",
        max_length=150,
        help_text="Настоящее имя юридического лица"
    )
    address = models.CharField(
        "Юридический адрес",
        max_length=150
    )
    inn = models.CharField(
        "ИНН компании",
        max_length=12,
        unique=True
    )
    kpp = models.CharField(
        "КПП компании",
        max_length=9
    )
    ogrn = models.CharField(
        "ОГРН компании",
        max_length=15
    )
    bank_name = models.CharField(
        "Название банка",
        max_length=150,
        help_text="Название банка с расчетным счетом"
    )
    bik = models.CharField(
        "БИК компании",
        max_length=11
    )
    rs = models.CharField(
        "Расчетный счет",
        max_length=20
    )
    ks = models.CharField(
        "Корреспондентский счет",
        max_length=20,
        help_text="Корреспондентский счет (счет, открываемый банковской организацией в подразделении самого банка)"
    )
    tax_method = models.IntegerField(
        help_text="Система налогооблажения"
    )

    # OTHER
    is_global = models.CharField(
        max_length=1,
        default="f",
        help_text="Доступно ли для хранения в глобальном сервисе / Нужна синхронизация данных"
    )
    is_visible = models.CharField(
        max_length=1,
        default="f",
        help_text="Доступно ли для планирования в цепочке с другими услугами в глобальном сервисе"
    )

    # FK
    repr = models.ForeignKey(
        get_user_model(),
        blank=True,
        verbose_name="Представитель организации",
        on_delete=models.DO_NOTHING,
        help_text="Уникальный идентификатор представителя компании"
    )
    ticket_status = models.ForeignKey(
        Ticket,
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        help_text="ID заявки, привязанной к текущей записи",
        verbose_name="Связанный тикет"
    )

    def deactivate(self):
        self.status = "deactivated"
        self.save()

    class Meta:
        db_table = "company"
        verbose_name = "Компания"
        verbose_name_plural = "Компании"
